; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			
; SLEEP+INT0+INT1 (externer Interrupt) konfigurieren						
;																			
; MCUCR => SE SM2 SM1 SM0 ISC11 ISC10 ISC01 ISC00 							
; SM2 SM1 SM0 Sleep Mode													
;*0   0   0   Idle															
; 0   0   1   ADC Noise Reduction											
; 0   1   0   Power-down													
; 0   1   1   Power-save													
; 1   0   0   Reserved														
; 1   0   1   Reserved														
; 1   1   0   Standby(1)													
;																			
;ISC11 ISC10 Description													
; 0 	0 	The low level of INT1 generates an interrupt request.			
; 0 	1 	Any logical change on INT1 generates an interrupt request.		
; 1 	0 	The falling edge of INT1 generates an interrupt request.		
; 1 	1 	The rising edge of INT1 generates an interrupt request.			
;																			
;ISC01 ISC00 Description													
; 0 	0 	The low level of INT0 generates an interrupt request.			
; 0 	1 	Any logical change on INT0 generates an interrupt request.		
;*1 	0 	The falling edge of INT0 generates an interrupt request.		
; 1 	1 	The rising  edge of INT0 generates an interrupt request.		
;																			
INT_SLEEP_INITIALISIEREN:

	ldi temp,(1<<SE)|(0<<SM2)|(0<<SM1)|(0<<SM0)|(0<<ISC11)|(0<<ISC10)|(1<<ISC01)|(0<<ISC00) 
	out MCUCR,temp

	cbi (DATA_DDR) ,(DATA_PIN_NR)
	sbi (DATA_PORT),(DATA_PIN_NR)
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; INT0+INT1 (externer Interrupt)aktivieren										
;																				
; GICR => INT1 INT0 - - - - IVSEL IVCE 											
; General Interrupt Control														
;																				
INT_AKTIVIEREN:
	ldi temp, (0<<INT1)|(1<<INT0)|(0<<IVSEL)|(0<<IVCE)   
    out GICR, temp 
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; RESET 20ms
EXT_INT0:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push ZL
	push ZH
;-------------------------------------------------------------------------------
	ldi ZL,low (adr_SERIELL_DATA)
	ldi ZH,high(adr_SERIELL_DATA)
	LDS temp,(adr_SERIELL_BYTE)
	add ZL,temp
	adc ZH,NULL
;-------------------------------------------------------------------------------
	clr temp
EXT_INT0_s:
	inc temp
	rcall WAIT_100us
	SBIS (DATA_PIN),(DATA_PIN_NR) 				; Skip if Bit...
	rjmp EXT_INT0_s
;-------------------------------------------------------------------------------
	cpi temp,50		; Reset ?	
	brlo EXT_INT0_HL
;-------------------------------------------------------------------------------
; Reset																			
	STS(adr_SERIELL_RESET),temp
	STS(adr_SERIELL_BIT) ,NULL
	STS(adr_SERIELL_BYTE),NULL
	ldi temp,255
EXT_INT0_CLEAR:
	ST Z+,NULL
	dec temp
	brne EXT_INT0_CLEAR
	rjmp EXT_INT0_EXIT
;-------------------------------------------------------------------------------
; H/L																			
EXT_INT0_HL:
	cpi temp,3			; H/L ?		
	brsh EXT_INT0_H
;-------------------------------------------------------------------------------
; L																				
	LD temp,Z	
	CLC 		; Clear Carry		
	ROR temp 	; rechts ber Carry	
	ST Z,temp

	LDS temp,(adr_SERIELL_BIT)
	inc temp
	STS(adr_SERIELL_BIT),temp
	cpi temp,8
	brsh EXT_INT0_BYTE
	rjmp EXT_INT0_EXIT
;-------------------------------------------------------------------------------
; H																				
EXT_INT0_H:
	LD temp,Z	
	SEC 		; Set Carry			
	ROR temp 	; rechts ber Carry	
	ST Z,temp

	LDS temp,(adr_SERIELL_BIT)
	inc temp
	STS(adr_SERIELL_BIT),temp
	cpi temp,8
	brsh EXT_INT0_BYTE
	rjmp EXT_INT0_EXIT
;-------------------------------------------------------------------------------
; BYTE-Counter																	
EXT_INT0_BYTE:
	LDS temp,(adr_SERIELL_BYTE)
	inc temp
	STS(adr_SERIELL_BYTE),temp
	STS(adr_SERIELL_BIT),NULL
	rjmp EXT_INT0_EXIT
;-------------------------------------------------------------------------------
EXT_INT0_EXIT:
	rcall EXT_AUSW
	pop ZH
	pop ZL
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 			
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
EXT_AUSW:
	LDS temp,(adr_SERIELL_BYTE)
	cpi temp,6
	breq EXT_AUSW_RUN
	LED_GRUEN_OFF
	ret
EXT_AUSW_RUN:
	LDS temp, (adr_SERIELL_DATA+5)
	cpi temp,'#'
	brne EXT_AUSW_ERR
	LED_GRUEN_ON
	ret
EXT_AUSW_ERR:
	LED_GRUEN_OFF
	LED_ROT_ON
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

